using System;
using System.Drawing;
using System.Windows.Forms;

public delegate void MoveCallback(int row,int column);

public class Board : UserControl
{
	public const int Side=40;
	public const int pad=Side/5;
	public NumberedButton[] buttons;
	private int boardSize;
	
	public Board(Logic logic)
		: base()
	{
		this.Size=new Size(logic.Size*(Side+pad),logic.Size*(Side+pad));
		this.Visible=true;
		boardSize=logic.Size;
		
		buttons=new NumberedButton[boardSize*boardSize];
		
		// interesting events
		this.Moved+=new MoveCallback(logic.Action);
		logic.Update+=new StateChangeCallback(this.UpdateBoard);

		for(int i=0;i<boardSize;i++)
		{
			for(int j=0;j<boardSize;j++)
			{
				NumberedButton btn=new NumberedButton(i,j);
				btn.Number=logic[i,j];
				btn.Location=new Point(pad/2+j*(Side+pad),pad/2+i*(Side+pad));
				btn.Size=new Size(Side,Side);
				Controls.Add(btn);
				btn.Click+=new EventHandler(ButtonClicked);
				buttons[i*boardSize+j]=btn;
			}			
		}
	}

	protected override void OnPaint(PaintEventArgs e)
	{
		Graphics graphics=e.Graphics;
		if(!Visible) return;
		ControlPaint.DrawBorder3D(graphics, 
							new Rectangle(0,0,Size.Width,Size.Height)
							,Border3DStyle.Sunken);
	}

	private void ButtonClicked(Object sender,EventArgs args)
	{
		NumberedButton btn=(sender as NumberedButton);
		if(Moved != null)
		{
			Moved(btn.Row, btn.Column);
		}
	}

	public virtual void UpdateBoard(Logic logic)
	{
		for(int i=0;i<boardSize;i++)
		{
			for(int j=0;j<boardSize;j++)
			{
				buttons[i*boardSize+j].Number=logic[i,j];
			}
		}
	}


	public override Size DefaultSize
	{
		get
		{
			return new Size(100,100);
		}
	}
	
	public event MoveCallback Moved;

	private class NumberedButton : Button
	{
		private int number;
		private int row, col;
		public NumberedButton(int row,int col)
			: base()
		{
			this.row=row;
			this.col=col;
		}
		public int Number
		{
			get
			{
				return number;
			}
			set
			{
				number=value;
				if(number==0)
				{
					this.Visible=false;
				}
				else
				{
					this.Visible=true;
				}
				this.Text=number.ToString();
			}
		}

		public int Row
		{
			get
			{
				return row;
			}
		}

		public int Column
		{
			get
			{
				return col;
			}
		}
	}
}
