using System;

public delegate void StateChangeCallback(Logic logic);

[Flags]
public enum MoveDirection
{
	Up=0x01,
	Down=0x02,
	Left=0x04,
	Right=0x08,
	All=Up|Down|Left|Right
}

public class Logic
{
	private int size;
	private int[] data;
	private int moves;
	
	public Logic()
		: this(4)
	{
	}

	public Logic(int size)
	{
		this.size=size;
		moves=0;
		data=new int[size*size];
		for(int i=0;i<size*size;i++)
		{
			data[i]=i+1;
		}
		data[size*size-1]=0;
	}

	public virtual void Shuffle()
	{
		moves=0;
		//TODO: better shuffle
		for(int i=0;i<119;i++)
		{
			int r=Environment.TickCount % (size*size);
			int temp=data[r];
			for(int j=0;j<data.Length;j++)
			{
				if(data[j]==0) data[j]=temp;
			}
			data[r]=0;
		}
	}

	public virtual void Restart()
	{
		Shuffle();
		if(Update != null)
		{
			Update(this);
		}
	}

	public virtual void Cheat()
	{
		for(int i=0;i<data.Length;i++)
		{
			data[i]=i+1;
		}
		data[data.Length-2]=0;
		data[data.Length-1]=data.Length-1;
		if(Update!=null)
		{
			Update(this);
		}
	}
	
	public int Size
	{
		get
		{
			return size;
		}
	}

	public event StateChangeCallback Win;
	public event StateChangeCallback Update;

	public virtual bool CheckForWin()
	{
		for(int i=1;i<size*size-1;i++)
		{
			if(data[i]<data[i-1]) return false;
		}
		if(data[size*size-1]!=0)return false;
		
		if(Win != null)
		{
			Win(this);
		}
		return true;
	}

	public virtual void Action(int row,int col)
	{
		bool value=ActionInternal(row,col,false,MoveDirection.All);
		if(value && Update!=null)
		{
			Update(this);
		}
		CheckForWin();
	}

	private bool ActionInternal(int row, int col, bool force,
									MoveDirection direction)
	{
		int temp;
		if((direction & MoveDirection.Up)!=0 && 
			((force && this[row-1,col]!=-1) 	|| this[row-1,col]==0))
		{
			temp=this[row-1, col];
			this[row-1,col]=this[row,col];
			this[row,col]=temp;
			return true;
		}
		if((direction & MoveDirection.Down)!=0 && 
			((force && this[row+1,col]!=-1) 	|| this[row+1,col]==0))
		{
			temp=this[row+1, col];
			this[row+1,col]=this[row,col];
			this[row,col]=temp;
			return true;
		}
		if((direction & MoveDirection.Left)!=0 && 
			((force && this[row,col-1]!=-1) 	|| this[row,col-1]==0))
		{
			temp=this[row, col-1];
			this[row,col-1]=this[row,col];
			this[row,col]=temp;
			return true;
		}
		if((direction & MoveDirection.Right)!=0 && 
			((force && this[row,col+1]!=-1) 	|| this[row,col+1]==0))
		{
			temp=this[row, col+1];
			this[row,col+1]=this[row,col];
			this[row,col]=temp;
			return true;
		}
		return false;
	}

	public virtual int this[int row, int col]
	{
		get
		{
			if(row<0 || row >=size) return -1;
			if(col<0 || col >=size) return -1;
			return data[row*size+col];
		}
		set
		{	
			if((row<0 || row >size) || (col<0 || col >size)) 
			{	
				throw new IndexOutOfRangeException("( "+row +" , " + col + ")");
			}
			data[row*size+col]=value;
		}
	}
	
}
