using System;
using System.Drawing;
using System.Windows.Forms;

public class MainForm : Form
{
	Logic logic;
	int moves=0;
	Label status;
	bool won=false;
	
	public MainForm(int side)
	{
		// Force the entire form to repaint when it is resized.
		SetStyle(ControlStyles.ResizeRedraw, true);

		// Set some initial form properties.
		Text = "8 Puzzle";
		
		logic=new Logic(side); // 4 x 4
		logic.Shuffle();
		
		Board board=new Board(logic);
		
		board.Location=new Point(Board.Side,Board.Side);
		Controls.Add(board);
		Width=board.Width+2*Board.Side;
		Height=board.Height+2*Board.Side;

		Button restart=new Button();
		restart.Location=new Point(5,5);
		restart.Text="Restart";
		restart.Click+=new EventHandler(RestartClicked);
		Controls.Add(restart);

		Button cheat=new Button();
		cheat.Location=new Point(10+restart.Width,5);
		cheat.Text="Cheat";
		cheat.Enabled=false;
		cheat.Click+=new EventHandler(CheatClicked);
		Controls.Add(cheat);

		board.Moved+=new MoveCallback(CountMoves);
		logic.Win+=new StateChangeCallback(YouWon);
		
		// Create a label and dock it to the bottom.
		status = new Label();
		status.Text = "Moves: 0";
		status.BackColor = Color.White;
		status.Dock = DockStyle.Bottom;
		Controls.Add(status);
	}

	private void CheatClicked(Object sender, EventArgs e)
	{
		logic.Cheat();
	}

	private void RestartClicked(Object sender, EventArgs e)
	{
		logic.Restart();
		moves=0;
		won=false;
	}

	private void CountMoves(int r,int c)
	{
		//TODO: clean up
		if(!won)
		{
			moves++;
			status.Text="Moves :"+moves;
		}
	}

	private void YouWon(Logic logic)
	{ 
		status.Text="You won in "+(moves+1)+" moves !";
		moves=0;
		won=true;
	}

	public static void Main(String []args)
	{
		int side=4;
		if(args.Length!=0)
		{
			side=Int32.Parse(args[0]);
		}
		MainForm form = new MainForm(side);
		Application.Run(form);
	}
}
